// @bun
// src/middleware.ts
import {userMiddlewarePath} from "@stacksjs/path";
async function importMiddlewares(directory) {
  return [directory];
}

class Middleware {
  name;
  priority;
  handle;
  constructor(data) {
    this.name = data.name;
    this.priority = data.priority;
    this.handle = data.handle;
  }
}
var middlewares = await importMiddlewares(userMiddlewarePath());
// src/request.ts
class Request {
  static instance;
  query = {};
  params = null;
  static getInstance() {
    if (!Request.instance) {
      Request.instance = new Request;
    }
    return Request.instance;
  }
  addQuery(url) {
    this.query = Object.fromEntries(url.searchParams);
  }
  get(element) {
    return this.query[element];
  }
  all() {
    return this.query;
  }
  has(element) {
    return element in this.query;
  }
  isEmpty() {
    return Object.keys(this.query).length === 0;
  }
  extractParamsFromRoute(routePattern, pathname) {
    const pattern = new RegExp(`^${routePattern.replace(/:(\w+)/g, (match2, paramName) => `(?<${paramName}>\\w+)`)}\$`);
    const match = pattern.exec(pathname);
    if (match?.groups)
      this.params = match.groups;
  }
  getParams(key) {
    return this.params ? this.params[key] || null : null;
  }
  getParamAsInt(key) {
    const value = this.params ? this.params[key] || null : null;
    return value ? Number.parseInt(value) : null;
  }
}
var request = new Request;
// src/server.ts
import process from "process";
import {log} from "@stacksjs/logging";
import {extname} from "@stacksjs/path";
async function serve(options = {}) {
  const hostname = options.host || "localhost";
  const port = options.port || 3000;
  const development = options.debug ? true : process.env.APP_ENV !== "production" && process.env.APP_ENV !== "prod";
  if (options.timezone)
    process.env.TZ = options.timezone;
  Bun.serve({
    hostname,
    port,
    development,
    async fetch(req) {
      return await serverResponse(req);
    }
  });
}
async function serverResponse(req) {
  log.info(`Incoming Request: ${req.method} ${req.url}`);
  log.info(`Headers: ${JSON.stringify(req.headers)}`);
  log.info(`Body: ${JSON.stringify(req.body)}`);
  const trimmedUrl = req.url.endsWith("/") && req.url.length > 1 ? req.url.slice(0, -1) : req.url;
  const url = new URL(trimmedUrl);
  addRouteParamsAndQuery(url);
  const routesList = await route.getRoutes();
  log.info(`Routes List: ${JSON.stringify(routesList)}`);
  log.info(`URL: ${JSON.stringify(url)}`);
  const foundRoute = routesList.find((route2) => {
    const pattern = new RegExp(`^${route2.uri.replace(/:\w+/g, "\\w+")}\$`);
    return pattern.test(url.pathname);
  });
  log.info(`Found Route: ${JSON.stringify(foundRoute)}`);
  if (!foundRoute)
    return new Response("Pretty 404 page coming soon", { status: 404 });
  await executeMiddleware(foundRoute);
  return await execute(foundRoute, req, { statusCode: foundRoute?.statusCode });
}
var addRouteParamsAndQuery = function(url) {
  if (!isObjectNotEmpty(url.searchParams))
    request.addQuery(url);
};
var executeMiddleware = function(route2) {
  const { middleware: middleware2 = null } = route2;
  if (middleware2 && middlewares && isObjectNotEmpty(middlewares)) {
    if (isString(middleware2)) {
    } else {
      middleware2.forEach(() => {
      });
    }
  }
};
async function execute(route2, req, { statusCode }) {
  if (!statusCode)
    statusCode = 200;
  if (route2?.method === "GET" && (statusCode === 301 || statusCode === 302)) {
    const callback = String(route2.callback);
    const response = Response.redirect(callback, statusCode);
    return await noCache(response);
  }
  if (route2?.method !== req.method)
    return new Response("Method not allowed", { status: 405 });
  if (isString(route2.callback) && extname(route2.callback) === ".html") {
    try {
      const fileContent = Bun.file(route2.callback);
      return await new Response(fileContent, {
        headers: { "Content-Type": "text/html" }
      });
    } catch (error) {
      return await new Response("Error reading the HTML file", { status: 500 });
    }
  }
  if (isString(route2.callback))
    return await new Response(route2.callback);
  if (isFunction(route2.callback)) {
    const result = route2.callback();
    return await new Response(JSON.stringify(result));
  }
  if (isObject(route2.callback))
    return await new Response(JSON.stringify(route2.callback));
  return await new Response("Unknown callback type.", { status: 500 });
}
var noCache = function(response) {
  response.headers.set("Cache-Control", "no-store, no-cache, must-revalidate, proxy-revalidate");
  response.headers.set("Pragma", "no-cache");
  response.headers.set("Expires", "0");
  return response;
};
var isString = function(val) {
  return typeof val === "string";
};
var isObjectNotEmpty = function(obj) {
  return Object.keys(obj).length > 0;
};
var isFunction = function(val) {
  return typeof val === "function";
};
var isObject = function(val) {
  return val !== null && typeof val === "object" && !Array.isArray(val);
};
// src/router.ts
import {log as log2} from "@stacksjs/logging";
import {path as p, projectStoragePath, routesPath} from "@stacksjs/path";
import {pascalCase} from "@stacksjs/strings";

class Router {
  routes = [];
  apiPrefix = "/api";
  groupPrefix = "";
  path = "";
  addRoute(method, uri, callback, statusCode) {
    const name = uri.replace(/\//g, ".").replace(/:/g, "");
    const pattern = new RegExp(`^${uri.replace(/:[a-zA-Z]+/g, (_match) => {
      return "([a-zA-Z0-9-]+)";
    })}\$`);
    let routeCallback;
    if (typeof callback === "string" || typeof callback === "object") {
      routeCallback = () => callback;
    } else {
      routeCallback = callback;
    }
    log2.debug(`Adding route: ${method} ${uri} with name ${name}`);
    this.routes.push({
      name,
      method,
      url: uri,
      uri,
      callback: routeCallback,
      pattern,
      statusCode,
      paramNames: []
    });
    return this;
  }
  async get(path4, callback) {
    this.path = this.normalizePath(path4);
    log2.debug(`Normalized Path: ${this.path}`);
    callback = await this.resolveCallback(callback);
    const uri = this.prepareUri(this.path);
    log2.debug(`Prepared URI: ${uri}`);
    return this.addRoute("GET", uri, callback, 200);
  }
  async email(path4) {
    path4 = pascalCase(path4);
    const emailModule = await import(p.userNotificationsPath(`${path4}.ts`));
    const callback = emailModule.default.handle;
    const uri = this.prepareUri(path4);
    this.addRoute("GET", uri, callback, 200);
    return this;
  }
  async health() {
    const healthModule = await import(p.userActionsPath("HealthAction.ts"));
    const callback = healthModule.default.handle;
    const path4 = healthModule.default.path ?? `${this.apiPrefix}/health`;
    this.addRoute("GET", path4, callback, 200);
    return this;
  }
  async job(path4) {
    path4 = pascalCase(path4);
    const jobModule = await import(p.userJobsPath(`${path4}Job.ts`.replace(/JobJob/, "Job")));
    const callback = jobModule.default.handle;
    path4 = this.prepareUri(path4);
    this.addRoute("GET", path4, callback, 200);
    return this;
  }
  async action(path4) {
    path4 = pascalCase(path4);
    let callback;
    try {
      const actionModule = await import(p.userActionsPath(`${path4}Action.ts`.replace(/ActionAction/, "Action")));
      callback = actionModule.default.handle;
    } catch (error) {
      try {
        const actionModule = await import(p.userActionsPath(`${path4}.ts`.replace(/ActionAction/, "Action")));
        callback = actionModule.default.handle;
      } catch (error2) {
        log2.error(`Could not find action module for path: ${path4}`);
        return this;
      }
    }
    path4 = this.prepareUri(path4);
    this.addRoute("GET", path4, callback, 200);
    return this;
  }
  post(path4, callback) {
    path4 = this.prepareUri(path4);
    this.addRoute("POST", path4, callback, 201);
    return this;
  }
  view(path4, callback) {
    this.addRoute("GET", path4, callback, 200);
    return this;
  }
  redirect(path4, callback, _status) {
    this.addRoute("GET", path4, callback, 302);
    return this;
  }
  delete(path4, callback) {
    this.addRoute("DELETE", path4, callback, 204);
    return this;
  }
  patch(path4, callback) {
    this.addRoute("PATCH", path4, callback, 202);
    return this;
  }
  put(path4, callback) {
    this.addRoute("PUT", path4, callback, 202);
    return this;
  }
  group(options, callback) {
    if (typeof options === "string")
      options = options.startsWith("/") ? options.slice(1) : options;
    let cb;
    this.prepareGroupPrefix(options);
    if (typeof options === "function") {
      cb = options;
      options = {};
    }
    if (!callback)
      throw new Error("Missing callback function for your route group.");
    cb = callback;
    const { prefix, middleware: middleware2 = [] } = options;
    const originalRoutes = this.routes;
    this.routes = [];
    cb();
    this.routes.forEach((r) => {
      r.uri = `${prefix}${r.uri}`;
      if (middleware2.length)
        r.middleware = middleware2;
      originalRoutes.push(r);
      return this;
    });
    this.routes = originalRoutes;
    return this;
  }
  name(name) {
    this.routes[this.routes.length - 1].name = name;
    return this;
  }
  middleware(middleware2) {
    this.routes[this.routes.length - 1].middleware = middleware2;
    return this;
  }
  prefix(prefix) {
    this.routes[this.routes.length - 1].prefix = prefix;
    return this;
  }
  async getRoutes() {
    await import(routesPath("api.ts"));
    await import(projectStoragePath("framework/orm/routes.ts"));
    return this.routes;
  }
  setGroupPrefix(prefix, options = {}) {
    if (prefix !== "") {
      prefix = `/${this.groupPrefix}/${prefix}`.replace(/\/\//g, "/");
      this.groupPrefix = prefix;
      return;
    }
    const effectiveOptions = typeof options === "object" ? options : {};
    this.groupPrefix = effectiveOptions.prefix ?? prefix ?? "";
  }
  prepareGroupPrefix(options) {
    if (this.groupPrefix !== "" && typeof options !== "string") {
      this.setGroupPrefix(this.groupPrefix, options);
      return;
    }
    if (typeof options === "string") {
      this.setGroupPrefix(options);
      return;
    }
    this.setGroupPrefix("", options);
    return;
  }
  async resolveCallback(callback) {
    if (callback instanceof Promise) {
      const actionModule = await callback;
      return actionModule.default;
    }
    if (typeof callback === "string")
      return await this.importCallbackFromPath(callback, this.path);
    return await callback;
  }
  async importCallbackFromPath(callbackPath, originalPath) {
    let modulePath = callbackPath;
    let importPathFunction = p.appPath;
    if (callbackPath.startsWith("../"))
      importPathFunction = p.routesPath;
    if (modulePath.includes("OrmAction"))
      importPathFunction = p.projectStoragePath;
    modulePath = modulePath.endsWith(".ts") ? modulePath.slice(0, -3) : modulePath;
    let actionModule = null;
    if (modulePath.includes("OrmAction")) {
      actionModule = await import(importPathFunction(`/framework/orm/${modulePath}.ts`));
    } else {
      actionModule = await import(importPathFunction(`${modulePath}.ts`));
    }
    const newPath = actionModule.default.path ?? originalPath;
    this.updatePathIfNeeded(newPath, originalPath);
    return await actionModule.default.handle();
  }
  normalizePath(path4) {
    return path4.endsWith("/") ? path4.slice(0, -1) : path4;
  }
  prepareUri(path4) {
    if (path4.startsWith("/"))
      path4 = path4.slice(1);
    path4 = `${this.apiPrefix}${this.groupPrefix}/${path4}`;
    console.log(path4);
    return path4.endsWith("/") ? path4.slice(0, -1) : path4;
  }
  updatePathIfNeeded(newPath, originalPath) {
    if (newPath !== originalPath) {
      this.path = newPath;
    }
  }
}
var route = new Router;
export {
  serverResponse,
  serve,
  route,
  request,
  middlewares,
  Router,
  Request,
  Middleware
};
